@@ -61,7 +61,7 @@ pub trait Server: Send + 'static {
     fn make_routes(&self) -> Routes;
 
     /// Combines this server with another server.
-    fn combine_with<Other, OtherSvc>(self, other: Other) -> ServerStack<Other, Self>
+    fn combine_with<Other>(self, other: Other) -> ServerStack<Other, Self>
     where
         Other: Server,
         Self: Sized,
@@ -126,12 +126,12 @@ where
 
 impl<L, S> Server for LayeredServer<L, S>
 where
-    L: Layer<Handler, Service = Handler> + Send + 'static,
+    L: Layer<Handler, Service = Handler> + Clone + Send + 'static,
     S: Server,
 {
     fn make_routes(&self) -> Routes {
         let rb = Server::make_routes(&self.inner);
-        rb.layer(&self.layer)
+        rb.layer_all(self.layer.clone())
     }
 }
 
